# Copyright (c) 2019-2021 - for information on the respective copyright owner
# see the NOTICE file and/or the repository
# https://github.com/boschresearch/pylife
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import numpy as np
import pandas as pd
import pdb
import pytest
import os
import sys

from . import data
from .data import collective

import pylife.strength.miner as miner
import pylife.strength.meanstress
from pylife.strength import miner
from pylife.stress.rainflow import *
from pylife.stress.timesignal import TimeSignalGenerator


# parameters for the example given in Haibach2006
sn_curve_parameters_haibach = {
    "ND": 1E6,
    "k_1": 4,
    "SD": 100
}
# parameters for the example given in Haibach2006
sn_curve_parameters_elementar = {
    "ND": 2E6,
    "k_1": 7,
    "SD": 125
}


@pytest.fixture(scope="module")
def miner_base():
    return miner.MinerBase(ND=10**6, k_1=5, SD=200)


@pytest.fixture(scope="module")
def miner_elementar():
    return miner.MinerElementar(ND=10**6, k_1=6, SD=200)


@pytest.fixture(scope="function")
def miner_haibach():
    m = miner.MinerHaibach(ND=10**6, k_1=6, SD=200)
    m.setup(collective)
    return m


def generate_transformed_pylife_collective():
    tsgen = TimeSignalGenerator(10, {'number': 50,
                                     'amplitude_median': 1.0, 'amplitude_std_dev': 0.5,
                                     'frequency_median': 4, 'frequency_std_dev': 3,
                                     'offset_median': 0, 'offset_std_dev': 0.4},
                                None, None)

    y = tsgen.query(2000000)

    rfc = ThreePointDetector(LoopValueRecorder()).process(y)
    rfm = rfc.recorder.matrix_series(128)

    transformed = rfm.meanstress_hist.FKM_goodman(pd.Series({'M': 0.3, 'M2': 0.1}), R_goal=-1)

    return transformed


@pytest.fixture(scope="module")
def transformed_pylife_collective():
    return generate_transformed_pylife_collective()


def test_get_accumulated_from_relative_collective():
    coll_acc = miner.get_accumulated_from_relative_collective(
        data.coll_haibach_mod_rel
    )
    assert np.allclose(data.coll_haibach_mod_acc,
                       coll_acc
                       )


class TestMinerBase:
    coll = collective

    def test__parse_collective_splitting_of_array(self, miner_base):
        miner_base._parse_collective(self.coll)

#        assert np.allclose(miner_base.S_collective, self.coll[:, 0])
#        assert np.allclose(miner_base.N_collective_accumulated, self.coll[:, 1])
        assert np.allclose(miner_base.S_collective, data.S_collective_normalized)
        assert np.allclose(miner_base.N_collective_accumulated, data.N_collective_accumulated)

    def test__parse_collective_relative_cycles(self, miner_base):
        miner_base._parse_collective(self.coll)

        assert np.allclose(miner_base.N_collective_accumulated, data.N_collective_accumulated)

    def test__parse_collective_relative(self, miner_base):
        miner_base._parse_collective(self.coll)

        assert np.allclose(miner_base.collective_relative,
                           np.stack((data.S_collective_normalized, data.N_collective_relative), axis=1))

    def test_calc_A_not_derived_class_fail(self, miner_base):
        with pytest.raises(NotImplementedError):
            miner_base.calc_A(self.coll)

    def test_pylife_collective_transformation(self, miner_base,
                                              transformed_pylife_collective):
        coll = transformed_pylife_collective
        parsed = miner_base._transform_pylife_collective(coll)
        # load classes have to be in ascending order
        assert np.all(np.diff(parsed[:, 0]) > 0 )
        # the frequencies are turned into accumulated frequencies in descending order
        assert np.all(np.diff(parsed[:, 1]) <= 0 )

    def test_parse_pylife_collective(self, miner_base,
                                     transformed_pylife_collective):
        coll = transformed_pylife_collective
        miner_base.setup(coll)


class TestMinerElementar:
    coll = collective

    def test_setup(self, miner_elementar):
        miner_elementar.setup(self.coll)

        H0 = self.coll[0, 1]
        np.testing.assert_almost_equal(H0, 190733.0)
        np.testing.assert_almost_equal(H0, miner_elementar.H0)
        # calculation of Zeitfestigkeitsfaktor
        # ND = 10**6
        # k = 6
        z = 1.3180413239445 # = (ND / H0)**(1. / k)
        np.testing.assert_almost_equal(miner_elementar.zeitfestigkeitsfaktor_collective, z)

    def test_A(self, miner_elementar):
        miner_elementar.setup(self.coll)

        # test requires k = 6
        A = miner_elementar.calc_A(self.coll)
        np.testing.assert_almost_equal(A, 862.99608075)

    def test_d_m(self, miner_elementar):
        if miner_elementar.A is None:
            miner_elementar.calc_A(self.coll)

        d_m = miner_elementar.effective_damage_sum(miner_elementar.A)
        d_m_test = 0.36900120961677296
        np.testing.assert_almost_equal(d_m, d_m_test)

    def test_d_m_limits(self, miner_elementar):
        # lower limit for d_m = 1
        assert miner_elementar.effective_damage_sum(A=16) == 1.0
        # upper limit for d_m = 0.3
        assert miner_elementar.effective_damage_sum(A=1975.308641975309) == 0.3

    def test_N_predict(self, miner_elementar):
        load_level = 400
        A = miner_elementar.calc_A(self.coll)
        N_woehler_load_level = (
            miner_elementar._woehler_curve.ND * (load_level / miner_elementar._woehler_curve.SD)**(-miner_elementar._woehler_curve.k_1)
        )
        N_predict = N_woehler_load_level * A
        np.testing.assert_almost_equal(miner_elementar.N_predict(load_level, A), N_predict)
        np.testing.assert_almost_equal(miner_elementar.N_predict(load_level), N_predict)

    def test_damage_accumulation_validation(self):
        """The test uses data from the book Haibach2006

        See miner.py for reference.
        The examples can be found on page 271.
        """
        coll = data.coll_elementar_acc
        load_level = coll[:, 0].max()
        expected_N = 2167330
        sn_curve_parameters = sn_curve_parameters_elementar
        miner_elementar = miner.MinerElementar(**sn_curve_parameters)
        miner_elementar.setup(coll)
        # some rounding is assumed in the example from the book
        # so in respect to millions of cycles a small neglectable tolerance is accepted
        np.testing.assert_approx_equal(expected_N, miner_elementar.N_predict(load_level), significant=6)


class TestMinerHaibach:
    def test_calc_A_no_collective_specified(self):
        m = miner.MinerHaibach(ND=10**6, k_1=5, SD=200)
        with pytest.raises(RuntimeError):
            m.calc_A(load_level=240)

    def test_calc_A_load_level_smaller_ND(self, miner_haibach):
        A = miner_haibach.calc_A(load_level=(miner_haibach._woehler_curve.SD / 2.))
        assert A == np.inf

    def test_calc_A_split_damage_regions(self, miner_haibach):
        load_level = 400
        miner_haibach.calc_A(load_level=load_level)

    def test_calc_A(self, miner_haibach):
        load_level = 400
        A = miner_haibach.calc_A(load_level)
        np.testing.assert_almost_equal(A, 1000.342377197)

    def test_N_predict(self, miner_haibach):
        load_level = 400
        N_woehler_load_level = (
            miner_haibach._woehler_curve.ND * (load_level / miner_haibach._woehler_curve.SD)**(-miner_haibach._woehler_curve.k_1)
        )
        A = miner_haibach.calc_A(load_level)
        N_predict = N_woehler_load_level * A
        np.testing.assert_almost_equal(miner_haibach.N_predict(load_level, A), N_predict)
        np.testing.assert_almost_equal(miner_haibach.N_predict(load_level), N_predict)

    @pytest.mark.parametrize("load_level, predicted_N", [
        (100, 935519000),
        (150, 61605400),
        (200, 11994100),
        (300, 1761820),
        (400, 505981),
        (600, 99382),
        (800, 31103)
    ])
    def test_damage_accumulation_validation(self, load_level, predicted_N):
        """The test uses data from the book Haibach2006

        See miner.py for reference.
        The examples can be found on page 292.
        """
        coll = data.coll_haibach_mod_acc
        sn_curve_parameters = sn_curve_parameters_haibach
        miner_haibach = miner.MinerHaibach(**sn_curve_parameters)
        miner_haibach.setup(coll)
        # some rounding is assumed in the example from the book
        # so in respect to millions of cycles a small neglectable tolerance is accepted
        np.testing.assert_approx_equal(predicted_N, miner_haibach.N_predict(load_level), significant=5)

    def test_N_predict_inf_rule(self):
        coll = data.coll_haibach_mod_acc
        sn_curve_parameters = sn_curve_parameters_haibach
        miner_haibach = miner.MinerHaibach(**sn_curve_parameters)
        miner_haibach.setup(coll)
        load_level = 50
        expected_N_100MPa = 935519000
        N_pred_ignored = miner_haibach.N_predict(load_level, ignore_inf_rule=True)
        N_pred_not_ignored = miner_haibach.N_predict(load_level, ignore_inf_rule=False)
        assert N_pred_not_ignored == np.inf
        assert N_pred_ignored > expected_N_100MPa



# Copyright (c) 2019-2021 - for information on the respective copyright owner
# see the NOTICE file and/or the repository
# https://github.com/boschresearch/pylife
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__author__ = "Johannes Mueller"
__maintainer__ = __author__

from .rambgood import RambergOsgood
from .true_stress_strain import *
from .woehlercurve import WoehlerCurve

__all__ = [
    'RambergOsgood',
    'true_stress',
    'true_strain',
    'true_fracture_stress',
    'true_fracture_strain',
    'WoehlerCurve'
]



# Copyright (c) 2019-2021 - for information on the respective copyright owner
# see the NOTICE file and/or the repository
# https://github.com/boschresearch/pylife
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__author__ = "Johannes Mueller"
__maintainer__ = __author__

import numpy as np
import pytest
from matplotlib.pyplot import axis

import pylife.utils.probability_data as PD
from pylife.utils.functions import rossow_cumfreqs
from pylife.utils.diagrams.probability_data import ProbabilityDataDiagram

@pytest.fixture
def pd():
    occurrences = [0.19144096, 2.32066184, 0.07425225, 1.32781569, 0.77430177,
                   4.65758492, 9.35734404, 0.72244567, 0.29089061, 1.57460074] # randomly generated
    probs = rossow_cumfreqs(len(occurrences))
    return PD.ProbabilityFit(probs, np.sort(occurrences))


def test_title_no_kw():
    diag = ProbabilityDataDiagram(None)
    assert diag.occurrences_name == 'Occurrences'
    assert diag.probability_name == 'Probability'
    assert diag.title == 'Probability plot'


def test_title_kw():
    diag = ProbabilityDataDiagram(None, occurrences_name='foo', probability_name='bar', title='baz')
    assert diag.occurrences_name == 'foo'
    assert diag.probability_name == 'bar'
    assert diag.title == 'baz'


def test_setters():
    diag = ProbabilityDataDiagram(None)
    diag.occurrences_name = 'foo'
    diag.probability_name = 'bar'
    diag.title = 'baz'
    assert diag.occurrences_name == 'foo'
    assert diag.probability_name == 'bar'
    assert diag.title == 'baz'


def test_plot_numerical_data(pd):
    diag = ProbabilityDataDiagram(pd)
    ax = diag.plot()
    xydata = ax.lines[0].get_xydata().T
    occurrences = xydata[0, :]
    percentiles = xydata[1, :]
    np.testing.assert_allclose(occurrences, pd.occurrences)
    np.testing.assert_allclose(percentiles, pd.percentiles)


def test_plot_axis(pd):
    diag = ProbabilityDataDiagram(pd)
    ax = diag.plot()
    assert ax.get_xscale() == 'log'
    assert ax.get_yscale() == 'linear'


def test_plot_titles(pd):
    diag = ProbabilityDataDiagram(pd)
    ax = diag.plot()
    assert ax.get_title() == "Probability plot"
    assert ax.get_ylabel() == "Probability"


def test_plot_ticks(pd):
    diag = ProbabilityDataDiagram(pd)
    ax = diag.plot()
    np.testing.assert_array_equal(ax.get_xticks(), np.logspace(-3., 3., 7))
    expected_yticks = np.array([-2.32634787, -1.64485363, -1.28155157, -0.84162123, 0.,
                                0.84162123, 1.28155157, 1.64485363, 2.32634787])
    np.testing.assert_allclose(ax.get_yticks(), expected_yticks, rtol=1e-4)



# Copyright (c) 2019-2021 - for information on the respective copyright owner
# see the NOTICE file and/or the repository
# https://github.com/boschresearch/pylife
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__author__ = "Johannes Mueller"
__maintainer__ = __author__

import cython
import numpy as np

from .general import AbstractDetector


class FKMDetector(AbstractDetector):
    """Rainflow detector as described in FKM non linear.

    The algorithm has been published by Clormann & Seeger 1985 and has
    been cited havily since.

    See the `here <subsection_FKM_>`_ in the demo for an example.

    Note
    ----
    This detector **does not** report the loop index.

    .. _subsection_FKM: ../demos/rainflow.ipynb#Algorithm-recommended-by-FKM-non-linear
    """

    def __init__(self, recorder):
        """Instantiate a FKMDetector.

        Parameters
        ----------
        recorder : subclass of :class:`.AbstractRecorder`
            The recorder that the detector will report to.
        """
        super().__init__(recorder)
        self._ir = 1
        self._residuals = []
        self._max_turn = 0.0

    @cython.locals(
        turns=cython.double[:],
        iz=cython.int, ir=cython.int,
        last0=cython.double, last1=cython.double,
        loop_assumed=cython.int,
        max_turn=cython.double)
    def process(self, samples):
        """Process a sample chunk.

        Parameters
        ----------
        samples : array_like, shape (N, )
            The samples to be processed

        Returns
        -------
        self : FKMDetector
            The ``self`` object so that processing can be chained
        """
        ir = self._ir
        max_turn = self._max_turn
        turns_index, turns = self._new_turns(samples)

        for current in turns:
            loop_assumed = True
            while loop_assumed:
                iz = len(self._residuals)
                loop_assumed = False
                if iz > ir:
                    last0 = self._residuals[-1]
                    last1 = self._residuals[-2]
                    if np.abs(current-last0) >= np.abs(last0-last1):
                        self._recorder.record_values(last1, last0)
                        self._residuals.pop()
                        self._residuals.pop()
                        if np.abs(last0) < max_turn and np.abs(last1) < max_turn:
                            loop_assumed = True
                elif iz == ir:
                    if np.abs(current) > max_turn:
                        ir += 1
            max_turn = max(np.abs(current), max_turn)
            self._residuals.append(current)

            self._ir = ir
            self._max_turn = max_turn

        return self



# Copyright (c) 2019-2021 - for information on the respective copyright owner
# see the NOTICE file and/or the repository
# https://github.com/boschresearch/pylife
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import numpy as np
import pandas as pd

from pylife.stress import histogram as hi


def test_combine_hist():
    hist1 = pd.Series(np.array([5, 10]), index=pd.interval_range(start=0, end=2))
    hist2 = pd.Series(np.array([12, 3, 20]), index=pd.interval_range(start=1, periods=3))

    expect_sum = pd.Series(np.array([5, 22, 3, 20]), index=pd.interval_range(start=0, periods=4))
    test_sum = hi.combine_hist([hist1, hist2], method='sum', nbins=4)
    pd.testing.assert_series_equal(test_sum, expect_sum, check_names=False)

    expect_min = pd.Series(np.array([5, 3]), index=pd.interval_range(
                                  start=0, end=4, periods=2))
    test_min = hi.combine_hist([hist1, hist2], method='min', nbins=2)
    pd.testing.assert_series_equal(test_min, expect_min, check_names=False)

    expect_max = pd.Series(np.array([12, 20]), index=pd.interval_range(
                                  start=0, end=4, periods=2))
    test_max = hi.combine_hist([hist1, hist2], method='max', nbins=2)
    pd.testing.assert_series_equal(test_max, expect_max, check_names=False)

    expect_mean = pd.Series(np.array([9., 11.5]), index=pd.interval_range(start=0, end=4, periods=2))
    test_mean = hi.combine_hist([hist1, hist2], method='mean', nbins=2)
    pd.testing.assert_series_equal(test_mean, expect_mean, check_names=False)

    expect_std = pd.Series(np.array([np.std(np.array([5, 10, 12])),
                                     np.std(np.array([3, 20]))]),
                           index=pd.interval_range(start=0, end=4, periods=2))
    test_std = hi.combine_hist([hist1, hist2], method='std', nbins=2)
    pd.testing.assert_series_equal(test_std, expect_std, check_names=False)



# Copyright (c) 2019-2021 - for information on the respective copyright owner
# see the NOTICE file and/or the repository
# https://github.com/boschresearch/pylife
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pylife.strength import helpers as hlp

import numpy as np
import pytest


# example collective
# array of classes
S = np.array([
    0.015625, 0.046875, 0.078125, 0.109375, 0.140625, 0.171875,
    0.203125, 0.234375, 0.265625, 0.296875, 0.328125, 0.359375,
    0.390625, 0.421875, 0.453125, 0.484375, 0.515625, 0.546875,
    0.578125, 0.609375, 0.640625, 0.671875, 0.703125, 0.734375,
    0.765625, 0.796875, 0.828125, 0.859375, 0.890625, 0.921875,
    0.953125, 0.984375, 1.015625, 1.046875, 1.078125, 1.109375,
    1.140625, 1.171875, 1.203125, 1.234375, 1.265625, 1.296875,
    1.328125, 1.359375, 1.390625, 1.421875, 1.453125, 1.484375,
    1.515625, 1.546875, 1.578125, 1.609375, 1.640625, 1.671875,
    1.703125, 1.734375, 1.765625, 1.796875, 1.828125, 1.859375,
    1.890625, 1.921875, 1.953125, 1.984375])
# array of continuous cycle count
N = np.array([
    190733.0, 190733.0, 181542.0, 169283.0, 153244.0, 124473.0,
    113752.0, 86802.0, 81117.0, 57718.0, 55726.0, 41750.0, 39033.0,
    30847.0, 28007.0, 23177.0, 20532.0, 17860.0, 15454.0, 13867.0,
    11601.0, 10765.0, 8831.0, 8449.0, 6682.0, 6504.0, 5044.0, 4967.0,
    3801.0, 3720.0, 2775.0, 2671.0, 2014.0, 1825.0, 1403.0, 1195.0,
    954.0, 717.0, 640.0, 418.0, 393.0, 257.0, 256.0, 152.0, 152.0,
    104.0, 104.0, 63.0, 62.0, 34.0, 32.0, 18.0, 18.0, 12.0, 9.0, 5.0,
    3.0, 3.0, 1.0, 1.0, 0, 0, 0, 0]
)

coll = np.stack((S, N), axis=1)
k = 6


def test_solidity_haibach():
    np.testing.assert_almost_equal(hlp.solidity_haibach(coll, k), 0.00115875)


def test_solidity_fkm():
    np.testing.assert_almost_equal(hlp.solidity_fkm(coll, k), 0.32408968)


class TestStressRelations:

    @pytest.mark.parametrize("amplitude, max_stress, R", [(200, 200, -1), (200, 400, 0)])
    def test_get_max_stress_from_amplitude(self, amplitude, max_stress, R):
        max_stress_calculated = hlp.StressRelations.get_max_stress_from_amplitude(
            amplitude=amplitude,
            R=R
        )
        np.testing.assert_almost_equal(max_stress, max_stress_calculated)

    @pytest.mark.parametrize("amplitude, mean_stress, R", [(200, 0, -1), (200, 200, 0)])
    def test_get_mean_stress_from_amplitude(self, amplitude, mean_stress, R):
        mean_stress_calculated = hlp.StressRelations.get_mean_stress_from_amplitude(
            amplitude=amplitude,
            R=R
        )
        np.testing.assert_almost_equal(mean_stress, mean_stress_calculated)


def test_irregularity_factor():
    """
    Consider the turning points series with 8 bins from 0 to 7:
    1 - 5 - 4 - 5 - 1 - 2 - 1 - 5 (mean bin = 3)

    The resulting rainflow matrix contains:
        - 5 -> 4
        - 5 -> 1 (2 mean value crossings: 1x upwards, 1x downwards)
        - 1 -> 2

    With the residuals:
        - 1 -> 5 (1 mean value crossing: 1x upwards)

    The two sided irregularity factor is the quotient of
        - the sum of mean value crossings (which is 3) and
        - the sum of turnings points (which is 8)

    So the irregularity factor is 3 / 8 = 0.375
    """
    rfm = np.array([
        #         to
        # 0  1  2  3  4  5  6  7
        [0, 0, 0, 0, 0, 0, 0, 0],  # 0
        [0, 0, 1, 0, 0, 0, 0, 0],  # 1
        [0, 0, 0, 0, 0, 0, 0, 0],  # 2 f
        [0, 0, 0, 0, 0, 0, 0, 0],  # 3 r
        [0, 0, 0, 0, 0, 0, 0, 0],  # 4 o
        [0, 1, 0, 0, 1, 0, 0, 0],  # 5 m
        [0, 0, 0, 0, 0, 0, 0, 0],  # 6
        [0, 0, 0, 0, 0, 0, 0, 0],  # 7
    ])

    residuals = np.array(
        [1, 5]
    )

    duplicated_residuals = np.array(
        [1, 1, 1, 5, 5]
    )

    # Calculated correctly
    np.testing.assert_almost_equal(hlp.irregularity_factor(rfm, residuals), 3./8, 4)

    # Decision bin inferred correctly
    np.testing.assert_almost_equal(hlp.irregularity_factor(rfm, decision_bin=None, residuals=residuals),
                                   hlp.irregularity_factor(rfm, decision_bin=3, residuals=residuals), 4)

    # Decision bin broadcast
    np.testing.assert_almost_equal(hlp.irregularity_factor(rfm, residuals, decision_bin=3.), 3./8, 4)
    with np.testing.assert_raises(ValueError):
        hlp.irregularity_factor(rfm, residuals, decision_bin="three")

    # Duplicates removed correctly
    np.testing.assert_almost_equal(hlp.irregularity_factor(rfm, residuals=residuals),
                                   hlp.irregularity_factor(rfm, residuals=duplicated_residuals), 4)



def test_irregularity_factor_non_square_rainflow():
    rfm = np.array([
        #         to
        # 0  1  2  3  4  5  6  7
        [0, 0, 0, 0, 0, 0, 0, 0],  # 0
        [0, 0, 1, 0, 0, 0, 0, 0],  # 1
        [0, 0, 0, 0, 0, 0, 0, 0],  # 2 f
        [0, 0, 0, 0, 0, 0, 0, 0],  # 3 r
        [0, 0, 0, 0, 0, 0, 0, 0],  # 4 o
        [0, 1, 0, 0, 1, 0, 0, 0],  # 5 m
        [0, 0, 0, 0, 0, 0, 0, 0],  # 6
    ])

    with pytest.raises(ValueError, match=r"Rainflow matrix must be square shaped in order to calculate the irregularity factor."):
        hlp.irregularity_factor(rfm)



# Copyright (c) 2019-2021 - for information on the respective copyright owner
# see the NOTICE file and/or the repository
# https://github.com/boschresearch/pylife
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest
import pandas as pd
import numpy as np

import pylife.stress.stresssignal


def test_voigt():
    df = pd.DataFrame({'S11': [1.0], 'S22': [1.0], 'S33': [1.0],
                       'S12': [1.0], 'S13': [1.0], 'S23': [1.0]})
    df.voigt


def test_voigt_fail():
    df = pd.DataFrame({'S11': [1.0], 'S22': [1.0], 'S33': [1.0],
                       'S12': [1.0], 'S31': [1.0], 'S23': [1.0]})
    with pytest.raises(AttributeError, match=r'^StressTensorVoigt.*Missing S13'):
        df.voigt


def test_cyclic_sigma_a_sigma_m():
    df = pd.DataFrame({'sigma_a': [1.0], 'sigma_m': [1.0]})
    df.cyclic_stress


def test_cyclic_only_sigma_a():
    df = pd.DataFrame({'sigma_a': [1.0]})
    np.testing.assert_equal(df.cyclic_stress.to_pandas()['sigma_m'].to_numpy(), np.zeros_like(df['sigma_a'].to_numpy()))


def test_cyclic_sigma_a_R():
    df = pd.DataFrame({'sigma_a': [1.0, 1.0], 'R': [0.0, -1.0]})
    np.testing.assert_equal(df.cyclic_stress.to_pandas()['sigma_m'].to_numpy(), [1.0, 0.0])


@pytest.mark.parametrize('R, sigma_m_check', [
    (0.0, 1.0),
    (-1.0, 0.0)
])
def test_cyclic_sigma_constant_R(R, sigma_m_check):
    df = pd.DataFrame({'sigma_a': [1.0], 'R': [R]})
    np.testing.assert_equal(df.cyclic_stress.to_pandas()['sigma_m'].to_numpy(), sigma_m_check)


def test_cyclic_no_sigma_a():
    df = pd.DataFrame({'sigma_m': [0.0]})
    with pytest.raises(AttributeError):
        df.cyclic_stress
